package alatar.components
{
    import flash.events.Event;
    
    import mx.controls.Image;
    import mx.core.IButton;
    
    import spark.components.Group;
    import spark.components.supportClasses.ButtonBase;
    
    [Style(name="accentColor", type="uint", format="Color", inherit="yes", theme="spark")]
    
    [Exclude(name="textAlign", kind="style")]
    
    public class IconButton extends ButtonBase implements IButton
    {
        [SkinPart(required="false")]
        public var iconGroup:Group;
        
        protected var iconImage:Image;
        
        public function IconButton()
        {
            super();
        }
        
        private var _icon:Object;
        
        public function get icon():Object
        {
            return _icon;
        }
        
        public function set icon(value:Object):void
        {
            if (_icon == value)
                return;
            
            _icon = value;
            
            if (!iconImage)
            {
                iconImage = new Image();
            }
            
            iconImage.source = _icon;
            
            if (iconGroup && !iconGroup.contains(iconImage))
            {
                iconGroup.addElement(iconImage);
            }
        }
        
        override protected function partAdded(partName:String, instance:Object):void
        {
            super.partAdded(partName, instance);
            
            if (instance == iconGroup && iconImage)
                iconGroup.addElement(iconImage);
        }
        
        override protected function partRemoved(partName:String, instance:Object):void
        {
            super.partRemoved(partName, instance);
            
            if (instance == iconGroup)
                iconGroup.removeAllElements();
        }
        
        //*************************
        // IButton implementation
        //*************************
        
        private var _emphasized:Boolean = false;
        
        [Bindable("emphasizedChanged")]
        [Inspectable(category="General", defaultValue="false")]
        
        public function get emphasized():Boolean 
        { 
            return _emphasized;
        }
        
        public function set emphasized(value:Boolean):void 
        {
            if (value == _emphasized)
                return;
            
            _emphasized = value;
            emphasizeStyleName();  
            dispatchEvent(new Event("emphasizedChanged"));
        }
        
        override public function set styleName(value:Object):void
        {
            super.styleName = value;
            
            // We need to ensure to re-apply the emphasized style if appropriate.
            if (value == null || value is String)
            {
                if (!value || (_emphasized && value.indexOf(" emphasized") == -1))
                    emphasizeStyleName();
            }
        }
        
        private function emphasizeStyleName():void
        {
            var style:String = styleName is String ? styleName as String : "";
            
            if (!styleName || styleName is String)
            {
                if (_emphasized)
                    super.styleName = style + " emphasized";
                else 
                    super.styleName = style.split(" emphasized").join("");
            }   
        }
    }
}